/* styles.css */

body {
    font-family: Arial, sans-serif;
    background-color: #e0f7fa;
    padding: 20px;
}

#main-content {
    padding: 20px;
    background-color: #f0f8ff;
}

#graph-visualization {
    width: 100%;
    height: 700px;
    border: 2px solid #4CAF50;
    box-shadow: 0 4px 8px 0 rgba(0, 0, 0, 0.2);
    border-radius: 10px;
    overflow: auto;
}

#chat-interface {
    position: fixed;
    bottom: 10px;
    right: 10px;
    width: 350px;
    min-width: 350px;
    max-width:max-content;
    height: 400px;
    border: 1px solid #ccc;
    padding: 10px;
    background-color: #ffffff;
    overflow-y: auto;
    box-shadow: 0 4px 8px 0 rgba(0, 0, 0, 0.2);
    border-radius: 10px;
    resize: horizontal; /* Allow resizing */
    direction: ltr; /* Flip the resize handle to the left */
    overflow: auto; /* Ensure scrollbars appear if content overflows */
}

#user-input {
    width: calc(100% - 20px);
    padding: 10px;
    margin-bottom: 10px;
    border: 1px solid #4CAF50;
    border-radius: 5px;
}

#submit-button {
    background-color: #4CAF50;
    color: white;
    border: none;
    padding: 10px 20px;
    text-align: center;
    text-decoration: none;
    display: inline-block;
    font-size: 16px;
    margin: 4px 2px;
    cursor: pointer;
    border-radius: 5px;
}

#reset-button {
    background-color: #f44336;
    color: white;
    border: none;
    padding: 10px 20px;
    text-align: center;
    text-decoration: none;
    display: inline-block;
    font-size: 16px;
    margin: 4px 2px;
    cursor: pointer;
    border-radius: 5px;
    margin-left: 10px;
}

#chat-output {
    margin-top: 10px;
    padding: 10px;
    border: 1px solid #ddd;
    border-radius: 5px;
    background-color: #f9f9f9;
}

#download-button{
    background-color: #adaf4c;
    color: white;
    border: none;
    padding: 10px 20px;
    text-align: center;
    text-decoration: none;
    display: inline-block;
    font-size: 16px;
    margin: 4px 2px;
    cursor: pointer;
    border-radius: 5px;
    margin-left: 10px;
}